@@ -1,32 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "defaultpropertyprovider.h"
 
 #include "qbskitinformation.h"
 #include "qbsprojectmanagerconstants.h"
+#include "qbsprojectmanagertr.h"
 
 #include <coreplugin/messagemanager.h>
 #include <baremetal/baremetalconstants.h>
@@ -125,28 +104,29 @@ static QString targetPlatform(const ProjectExplorer::Abi &abi, const ProjectExpl
 
 static QStringList toolchainList(const ProjectExplorer::ToolChain *tc)
 {
-    QStringList list;
-    if (tc->typeId() == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID
-            || (tc->typeId() == Android::Constants::ANDROID_TOOLCHAIN_TYPEID
+    const Utils::Id type = tc->typeId();
+    if (type == ProjectExplorer::Constants::CLANG_TOOLCHAIN_TYPEID
+            || (type == Android::Constants::ANDROID_TOOLCHAIN_TYPEID
                 && tc->compilerCommand().toString().contains("clang"))) {
-        list << QLatin1String("clang") << QLatin1String("llvm") << QLatin1String("gcc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::GCC_TOOLCHAIN_TYPEID
-               || tc->typeId() == Android::Constants::ANDROID_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("gcc"); // TODO: Detect llvm-gcc
-    } else if (tc->typeId() == ProjectExplorer::Constants::MINGW_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("mingw") << QLatin1String("gcc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("clang-cl") << QLatin1String("msvc");
-    } else if (tc->typeId() == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("msvc");
-    } else if (tc->typeId() == BareMetal::Constants::IAREW_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("iar");
-    } else if (tc->typeId() == BareMetal::Constants::KEIL_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("keil");
-    } else if (tc->typeId() == BareMetal::Constants::SDCC_TOOLCHAIN_TYPEID) {
-        list << QLatin1String("sdcc");
+        return {"clang", "llvm", "gcc"};
     }
-    return list;
+    if (type == ProjectExplorer::Constants::GCC_TOOLCHAIN_TYPEID
+               || type == Android::Constants::ANDROID_TOOLCHAIN_TYPEID) {
+        return {"gcc"}; // TODO: Detect llvm-gcc
+    }
+    if (type == ProjectExplorer::Constants::MINGW_TOOLCHAIN_TYPEID)
+        return {"mingw", "gcc"};
+    if (type == ProjectExplorer::Constants::CLANG_CL_TOOLCHAIN_TYPEID)
+        return {"clang-cl", "msvc"};
+    if (type == ProjectExplorer::Constants::MSVC_TOOLCHAIN_TYPEID)
+        return {"msvc"};
+    if (type == BareMetal::Constants::IAREW_TOOLCHAIN_TYPEID)
+        return {"iar"};
+    if (type == BareMetal::Constants::KEIL_TOOLCHAIN_TYPEID)
+        return {"keil"};
+    if (type == BareMetal::Constants::SDCC_TOOLCHAIN_TYPEID)
+        return {"sdcc"};
+    return {};
 }
 
 static QString architecture(const ProjectExplorer::Abi &targetAbi)
@@ -352,7 +332,7 @@ QVariantMap DefaultPropertyProvider::autoGeneratedProperties(const ProjectExplor
         if (tcC && tcCxx && !cCompilerPath.isEmpty() && !cxxCompilerPath.isEmpty()
                 && cCompilerPath.absolutePath() != cxxCompilerPath.absolutePath()) {
             Core::MessageManager::writeFlashing(
-                tr("C and C++ compiler paths differ. C compiler may not work."));
+                Tr::tr("C and C++ compiler paths differ. C compiler may not work."));
         }
         data.insert(QLatin1String(CPP_TOOLCHAINPATH), mainFilePath.absolutePath().toString());
 
