@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "genericproposalmodel.h"
 
@@ -36,11 +14,12 @@
 #include <QHash>
 
 #include <algorithm>
+#include <utils/algorithm.h>
 
 using namespace TextEditor;
 
 QT_BEGIN_NAMESPACE
-uint qHash(const AssistProposalItem &item)
+auto qHash(const AssistProposalItem &item)
 {
     return qHash(item.text());
 }
@@ -304,7 +283,7 @@ void GenericProposalModel::filter(const QString &prefix)
     m_currentItems.clear();
     const QString lowerPrefix = prefix.toLower();
     const bool checkInfix = prefix.size() >= 3;
-    for (const auto &item : qAsConst(m_originalItems)) {
+    for (const auto &item : std::as_const(m_originalItems)) {
         const QString &text = item->filterText();
 
         // Direct match?
@@ -417,3 +396,13 @@ AssistProposalItemInterface *GenericProposalModel::proposalItem(int index) const
 {
     return m_currentItems.at(index);
 }
+
+int GenericProposalModel::indexOf(
+    const std::function<bool(AssistProposalItemInterface *)> &predicate) const
+{
+    for (int index = 0, end = m_currentItems.size(); index < end; ++index) {
+        if (predicate(m_currentItems.at(index)))
+            return index;
+    }
+    return -1;
+}
